@@ -1,21 +1,21 @@
 /*
- * Copyright (C) 2009 Debarshi Ray <debarshir@src.gnome.org>
- * Copyright (C) 2009 Nokia
+ * Copyright (C) 2009, Debarshi Ray <debarshir@src.gnome.org>
+ * Copyright (C) 2009, Nokia <ivan.frade@nokia.com>
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public
- * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
  *
- * This library is distributed in the hope that it will be useful,
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
- *
- * You should have received a copy of the GNU General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
  *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+ * 02110-1301, USA.
  */
 
 #include "config.h"
@@ -25,7 +25,8 @@
 #include <glib/gi18n.h>
 
 #include <libnautilus-extension/nautilus-file-info.h>
-#include <libtracker-client/tracker.h>
+
+#include <libtracker-client/tracker-client.h>
 
 #include "tracker-tags-utils.h"
 #include "tracker-tags-view.h"
@@ -39,6 +40,7 @@ struct _TrackerTagsViewPrivate {
 	GtkListStore *store;
 
 	GtkWidget *button_add;
+	GtkWidget *button_remove;
 	GtkWidget *entry;
 	GtkWidget *view;
 };
@@ -47,6 +49,7 @@ typedef struct {
 	TrackerTagsView *tv;
 	gchar *tag_id;
 	GtkTreeIter *iter;
+	gint items;
 	gboolean update;
 	gboolean selected;
 } TagData;
@@ -94,7 +97,7 @@ tracker_tags_view_init (TrackerTagsView *tv)
 {
 	tv->private = TRACKER_TAGS_VIEW_GET_PRIVATE (tv);
 
-	tv->private->tracker_client = tracker_connect (TRUE, G_MAXINT);
+	tv->private->tracker_client = tracker_client_new (TRACKER_CLIENT_ENABLE_WARNINGS, G_MAXINT);
 	tv->private->files = NULL;
 	tv->private->store = gtk_list_store_new (N_COLUMNS, 
 	                                         G_TYPE_INT,      /* Selection type */ 
@@ -109,7 +112,7 @@ tracker_tags_view_finalize (GObject *object)
 {
 	TrackerTagsView *tv = TRACKER_TAGS_VIEW (object);
 
-	tracker_disconnect (tv->private->tracker_client);
+	g_object_unref (tv->private->tracker_client);
 
 	g_list_foreach (tv->private->files, (GFunc) g_object_unref, NULL);
 	g_list_free (tv->private->files);
@@ -122,6 +125,7 @@ tag_data_new (gchar           *tag_id,
               GtkTreeIter     *iter,
               gboolean         update,
               gboolean         selected,
+              gint             items,
               TrackerTagsView *tv)
 {
 	TagData *td;
@@ -138,6 +142,7 @@ tag_data_new (gchar           *tag_id,
 		td->iter = NULL;
 	}
 
+	td->items = items;
 	td->update = update;
 	td->selected = selected;
 
@@ -173,6 +178,7 @@ tag_data_copy (TagData *td)
 		new_td->iter = NULL;
 	}
 
+	new_td->items = td->items;
 	new_td->update = td->update;
 	new_td->selected = td->selected;
 
@@ -320,22 +326,6 @@ tags_view_query_files_for_tag_id_cb (GPtrArray *result,
 
 			g_free (uri);
 		}
-	} else {
-		gchar *query;
-
-		g_debug ("Removing unused tag\n");
-
-		/* We automatically remove the tag */
-		query = g_strdup_printf ("DELETE { "
-		                         "  <%s> a nao:Tag "
-		                         "}",
-		                         td->tag_id);
-
-		tracker_resources_sparql_update_async (tv->private->tracker_client, 
-		                                       query, 
-		                                       tags_view_tag_removed_cb,
-		                                       tag_data_copy (td));
-		g_free (query);
 	}
 
 	g_debug ("Querying files with tag, in selection:%d, in total:%d, selected:%d\n", 
@@ -369,20 +359,20 @@ tags_view_query_files_for_tag_id_cb (GPtrArray *result,
 }
 
 static void
-tags_view_query_files_for_tag_id (TagData *td)                              
+tags_view_query_files_for_tag_id (TagData *td)
 {
 	gchar *query;
 
-	query = g_strdup_printf ("SELECT ?f "
-				 "WHERE {"
-				 "  ?f a rdfs:Resource ;"
-				 "  nao:hasTag <%s> ."
-				 "}",
-				 td->tag_id);
+	query = g_strdup_printf ("SELECT ?url "
+	                         "WHERE {"
+	                         "  ?urn a rdfs:Resource ;"
+	                         "  nie:url ?url ;"
+	                         "  nao:hasTag <%s> . "
+	                         "}", td->tag_id);
 
 	tracker_resources_sparql_query_async (td->tv->private->tracker_client,
-                                              query,
-                                              tags_view_query_files_for_tag_id_cb,
+	                                      query,
+	                                      tags_view_query_files_for_tag_id_cb,
 	                                      td);
 	g_free (query);
 }
@@ -405,7 +395,7 @@ tags_view_append_foreach (gpointer data,
 	                    COL_SELECTION, SELECTION_FALSE, 
 	                    -1);
 
-	td = tag_data_new (strv[0], &iter, FALSE, TRUE, tv);
+	td = tag_data_new (strv[0], &iter, FALSE, TRUE, 1, tv);
 	tags_view_query_files_for_tag_id (td);
 }
 
@@ -478,16 +468,20 @@ tags_view_model_update_cb (GError   *error,
 
 		if (!td->update) {
 			GtkTreeIter iter;
+			gchar *str;
 
 			g_debug ("Setting tag selection state to ON (new)\n");
 
+			str = g_strdup_printf ("%d", td->items);
 			gtk_list_store_append (tv->private->store, &iter);
 			gtk_list_store_set (tv->private->store, &iter,
 			                    COL_TAG_ID, td->tag_id,
 			                    COL_TAG_NAME, tag,
-			                    COL_TAG_COUNT, 0,
+			                    COL_TAG_COUNT, str,
+			                    COL_TAG_COUNT_VALUE, td->items,
 			                    COL_SELECTION, SELECTION_TRUE,
 			                    -1);
+			g_free (str);
 		} else if (td->selected) {
 			g_debug ("Setting tag selection state to ON\n");
 
@@ -517,22 +511,42 @@ static void
 tags_view_add_tag (TrackerTagsView *tv,
                    const gchar     *tag)
 {
-	gchar *query;
+	GString *query;
+	gint files;
 
 	gtk_widget_set_sensitive (tv->private->entry, FALSE);
 
-	if (g_list_length (tv->private->files) > 0) {
+	files = g_list_length (tv->private->files);
+
+	if (files > 0) {
 		GStrv files;
 		gchar *tag_escaped;
 		gchar *filter;
+		guint i;
+
+		query = g_string_new ("");
 
 		files = tracker_glist_to_string_list_for_nautilus_files (tv->private->files);
 		filter = tracker_tags_get_filter_string (files, NULL);
 		tag_escaped = tracker_tags_escape_sparql_string (tag);
 
-		query = g_strdup_printf ("INSERT { "
+		for (i = 0; files[i] != NULL; i++) {
+			g_string_append_printf (query, 
+			                        "INSERT { _:file a nie:DataObject ; nie:url '%s' } "
+			                        "WHERE { "
+			                        "  OPTIONAL {"
+			                        "     ?file a nie:DataObject ;"
+			                        "     nie:url '%s'"
+			                        "  } ."
+			                        "  FILTER (!bound(?file)) "
+			                        "} ",
+			                        files[i], files[i]);
+		}
+
+		g_string_append_printf (query,
+		                         "INSERT { "
 		                         "  _:tag a nao:Tag;"
-		                         "  nao:prefLabel %s ."
+		                         "  nao:prefLabel %s . "
 		                         "} "
 		                         "WHERE {"
 		                         "  OPTIONAL {"
@@ -542,11 +556,11 @@ tags_view_add_tag (TrackerTagsView *tv,
 		                         "  FILTER (!bound(?tag)) "
 		                         "} "
 		                         "INSERT { "
-		                         "  ?urn nao:hasTag ?id "
+		                         "  ?urn nao:hasTag ?label "
 		                         "} "
 		                         "WHERE {"
-		                         "  ?urn nie:isStoredAs ?f ."
-		                         "  ?id nao:prefLabel %s "
+		                         "  ?urn nie:url ?f ."
+		                         "  ?label nao:prefLabel %s "
 		                         "  %s "
 		                         "}",
 		                         tag_escaped,
@@ -558,15 +572,15 @@ tags_view_add_tag (TrackerTagsView *tv,
 		g_free (filter);
 		g_strfreev (files);
 	} else {
-		query = tracker_tags_add_query (tag);
+		query = g_string_new (tracker_tags_add_query (tag));
 	}
 
 	tracker_resources_sparql_update_async (tv->private->tracker_client,
-	                                       query,
+	                                       query->str,
 	                                       tags_view_model_update_cb,
-	                                       tag_data_new (NULL, NULL, FALSE, TRUE, tv));
+	                                       tag_data_new (NULL, NULL, FALSE, TRUE, files, tv));
 
-	g_free (query);
+	g_string_free (query, TRUE);
 }
 
 static void
@@ -622,33 +636,33 @@ tags_view_model_toggle_row (TrackerTagsView *tv,
 
 	if (selection) {
 		query = g_strdup_printf ("INSERT { "
-					 "  ?urn nao:hasTag ?tag "
-					 "} "
-					 "WHERE {"
-					 "  ?urn nie:isStoredAs ?f ." /* NB: ?f is used in filter. */
-					 "  ?tag nao:prefLabel %s ."
-					 "  %s "
-					 "}",
-					 tag_escaped, 
+		                         "  ?urn nao:hasTag ?label "
+		                         "} "
+		                         "WHERE {"
+		                         "  ?urn nie:url ?f ." /* NB: ?f is used in filter. */
+		                         "  ?label nao:prefLabel %s ."
+		                         "  %s "
+		                         "}", 
+		                         tag_escaped, 
 		                         filter);
 	} else {
 		TagData *td;
 
 		query = g_strdup_printf ("DELETE { "
-					 "  ?urn nao:hasTag ?tag "
-					 "} "
-					 "WHERE { "
-					 "  ?urn nie:isStoredAs ?f ." /* NB: ?f is used in filter. */
-					 "  ?tag nao:prefLabel %s ."
-					 "  %s "
-					 "}",
-					 tag_escaped, 
+		                         "  ?urn nao:hasTag ?label "
+		                         "} "
+		                         "WHERE { "
+		                         "  ?urn nie:url ?f ." /* NB: ?f is used in filter. */
+		                         "  ?label nao:prefLabel %s ."
+		                         "  %s "
+		                         "}",
+		                         tag_escaped, 
 		                         filter);
 		
 		/* Check if there are any files left with this tag and
 		 * remove tag if not.
 		 */
-		td = tag_data_new (id, &iter, FALSE, TRUE, tv);
+		td = tag_data_new (id, &iter, FALSE, TRUE, 1, tv);
 		tags_view_query_files_for_tag_id (td);
 	}
 
@@ -662,7 +676,7 @@ tags_view_model_toggle_row (TrackerTagsView *tv,
 	tracker_resources_sparql_update_async (tv->private->tracker_client, 
 	                                       query, 
 	                                       tags_view_model_update_cb, 
-	                                       tag_data_new (id, &iter, TRUE, selection, tv));
+	                                       tag_data_new (id, &iter, TRUE, selection, 1, tv));
 
 	g_free (id);
 	g_free (query);
@@ -724,6 +738,60 @@ tags_view_add_clicked_cb (GtkButton *button,
 	tags_view_add_tag (tv, tag);
 }
 
+static void
+tags_view_remove_tag (TrackerTagsView *tv,
+                      TagData         *td)
+{
+	gchar *query;
+
+	query = g_strdup_printf ("DELETE { "
+	                         "  <%s> a rdfs:Resource "
+	                         "}",
+	                         td->tag_id);
+
+	tracker_resources_sparql_update_async (tv->private->tracker_client, 
+	                                       query, 
+	                                       tags_view_tag_removed_cb,
+	                                       tag_data_copy (td));
+	g_free (query);
+}
+
+static void
+tags_view_remove_clicked_cb (GtkButton *button, 
+                             gpointer   user_data)
+{
+	TrackerTagsView *tv = user_data;
+	TagData *td;
+	GtkTreeIter iter;
+	GtkTreeSelection *select;
+	GtkTreeModel *model;
+	gchar *id;
+
+	select = gtk_tree_view_get_selection (GTK_TREE_VIEW (tv->private->view));
+
+	if (gtk_tree_selection_get_selected (select, &model, &iter)) {
+		gtk_tree_model_get (GTK_TREE_MODEL (tv->private->store), &iter, COL_TAG_ID, &id, -1);
+		td = tag_data_new (id, &iter, FALSE, TRUE, 1, tv);
+		tags_view_remove_tag (tv, td);
+		tag_data_free (td);
+	}
+}
+
+static void
+tags_view_model_row_selected_cb (GtkTreeSelection *selection,
+                                 gpointer         user_data)
+{
+	TrackerTagsView *tv = user_data;
+	GtkTreeIter iter;
+	GtkTreeModel *model;
+
+	if (gtk_tree_selection_get_selected (selection, &model, &iter)) {
+		gtk_widget_set_sensitive (GTK_WIDGET (tv->private->button_remove), TRUE);
+	} else {
+		gtk_widget_set_sensitive (GTK_WIDGET (tv->private->button_remove), FALSE);
+	}
+}
+
 static void
 tags_view_create_ui (TrackerTagsView *tv)
 {
@@ -782,6 +850,19 @@ tags_view_create_ui (TrackerTagsView *tv)
 	                  G_CALLBACK (tags_view_add_clicked_cb), 
 	                  tv);
 
+	tv->private->button_add = button;
+
+	button = gtk_button_new_from_stock (GTK_STOCK_REMOVE);
+	gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, TRUE, 0);
+
+	gtk_widget_set_sensitive (button, FALSE);
+
+	g_signal_connect (button, "clicked", 
+	                  G_CALLBACK (tags_view_remove_clicked_cb), 
+	                  tv);
+
+	tv->private->button_remove = button;
+
 	/* List */
 	scrolled_window = gtk_scrolled_window_new (NULL, NULL);
 	gtk_box_pack_start (GTK_BOX (tv), scrolled_window, TRUE, TRUE, 0);
@@ -803,8 +884,8 @@ tags_view_create_ui (TrackerTagsView *tv)
 
 	cell_renderer = gtk_cell_renderer_toggle_new ();
 	g_signal_connect (cell_renderer, "toggled",
-			  G_CALLBACK (tags_view_model_cell_toggled_cb),
-			  tv);
+	                  G_CALLBACK (tags_view_model_cell_toggled_cb),
+	                  tv);
 
 	gtk_tree_view_column_pack_start (column, cell_renderer, TRUE);
 	gtk_tree_view_column_set_cell_data_func (column, 
@@ -846,20 +927,23 @@ tags_view_create_ui (TrackerTagsView *tv)
 	                  G_CALLBACK (tags_view_model_row_activated_cb), 
 	                  tv);
 
+	g_signal_connect (selection, "changed", 
+	                  G_CALLBACK (tags_view_model_row_selected_cb), 
+	                  tv);
+
 	tracker_resources_sparql_query_async (tv->private->tracker_client,
-					      "SELECT ?u ?t "
-					      "WHERE {"
-					      "  ?u a nao:Tag ;"
-					      "  nao:prefLabel ?t ."
-					      "}",
-					      tags_view_add_tags_cb, tv);
+	                                      "SELECT ?urn ?label "
+	                                      "WHERE {"
+	                                      "  ?urn a nao:Tag ;"
+	                                      "  nao:prefLabel ?label . "
+	                                      "} ORDER BY ?label",
+	                                      tags_view_add_tags_cb, tv);
 
 	gtk_widget_show_all (GTK_WIDGET (tv));
 	gtk_widget_grab_focus (entry);
 
 	/* Save vars */
 	tv->private->entry = entry;
-	tv->private->button_add = button;
 	tv->private->view = view;
 }
 
