@@ -163,6 +163,7 @@ typedef struct {
 	gboolean preserve_attribute_present;
 	const gchar *uri;
 	GString *content;
+	gboolean title_already_set;
 } MsOfficeXMLParserInfo;
 
 typedef struct {
@@ -935,12 +936,21 @@ extract_msword_content (GsfInfile *infile,
 
 	while (TRUE) {
 		if (clx[i] == 2) {
+			/* Nice, a proper structure with contents, no need to
+			 * iterate more. */
 			lcb_piece_table = read_32bit (clx + (i + 1));
 			piece_table = clx + i + 5;
 			piece_count = (lcb_piece_table - 4) / 12;
 			break;
 		} else if (clx[i] == 1) {
-			i = i + 2 + clx[i + 1];
+			/* Oh, a PRC structure with properties of text, not
+			 * real text, so skip it */
+			guint16 GrpPrl_len;
+
+
+			GrpPrl_len = read_16bit (&clx[i+1]);
+			/* 3 is the length of clxt (1byte) and cbGrpprl(2bytes) */
+			i = i + 3 + GrpPrl_len;
 		} else {
 			break;
 		}
@@ -2001,8 +2011,14 @@ xml_text_handler_document_data (GMarkupParseContext  *context,
 		break;
 
 	case MS_OFFICE_XML_TAG_TITLE:
-		tracker_sparql_builder_predicate (info->metadata, "nie:title");
-		tracker_sparql_builder_object_unvalidated (info->metadata, text);
+		if (info->title_already_set) {
+			g_warning ("Avoiding additional title (%s) in MsOffice XML document '%s'",
+			           text, info->uri);
+		} else {
+			info->title_already_set = TRUE;
+			tracker_sparql_builder_predicate (info->metadata, "nie:title");
+			tracker_sparql_builder_object_unvalidated (info->metadata, text);
+		}
 		break;
 
 	case MS_OFFICE_XML_TAG_SUBJECT:
@@ -2113,6 +2129,7 @@ xml_read (MsOfficeXMLParserInfo *parser_info,
 	info.preserve_attribute_present = FALSE;
 	info.uri = parser_info->uri;
 	info.content = parser_info->content;
+	info.title_already_set = parser_info->title_already_set;
 
 	switch (type) {
 	case MS_OFFICE_XML_TAG_DOCUMENT_CORE_DATA: {
@@ -2304,6 +2321,7 @@ extract_msoffice_xml (const gchar          *uri,
 	info.preserve_attribute_present = FALSE;
 	info.uri = uri;
 	info.content = g_string_new ("");
+	info.title_already_set = FALSE;
 
 	context = g_markup_parse_context_new (&parser, 0, &info, NULL);
 
